<?php
/*
 +---------------------------------------------------------------------------------------+
 | Code PHP for Device/Connection Uptime                                                 |
 |                                                                                       |
 | Date : 19/11/2108                                                                     |
 | Coded By Oijkn                                                                        |
 +---------------------------------------------------------------------------------------+
 | Source :                                                                              |
 |                                                                                       |
 | https://docs.cacti.net/manual:087:3a_advanced_topics.3d_script_data_query_walkthrough |
 +---------------------------------------------------------------------------------------+
*/
 
/* do NOT run this script through a web browser */
if (!isset($_SERVER["argv"][0]) || isset($_SERVER['REQUEST_METHOD'])  || isset($_SERVER['REMOTE_ADDR'])) {
   die("<br><strong>This script is only meant to run at the command line.</strong>");
}
 
# deactivate http headers
$no_http_headers = true;

# include some cacti files for ease of use
include(dirname(__FILE__) . "/../include/global.php");
include(dirname(__FILE__) . "/../lib/snmp.php");
 
# define all OIDs we need for further processing
$oids = array(
        "index"          => ".1.3.6.1.2.1.11.32",
        "deviceUptime"   => ".1.3.6.1.4.1.17713.20.2.1.5.18",
        "connectionTime" => ".1.3.6.1.4.1.17713.20.2.1.1.6",
        "description"    => ".1.3.6.1.4.1.17713.20.2.1.4.2"
        );
$xml_delimiter          =  "!";
 
# all required input parms
$hostname       = $_SERVER["argv"][1];		# hostname/IP@
$cmd            = $_SERVER["argv"][2];		# one of: index/query/get

if (isset($_SERVER["argv"][3])) { $query_field = $_SERVER["argv"][3]; };
if (isset($_SERVER["argv"][4])) { $query_index = $_SERVER["argv"][4]; };
 
# get number of snmp retries from global settings
$snmp_retries   = read_config_option("snmp_retries");

# put your own community string here
$snmp_community = "public";			# community string
$snmp_version   = 2;				# snmp version
$snmp_port      = 161;				# snmp port
$snmp_timeout   = 2000;				# snmp timeout
$snmp_retries   = 3;				# snmp retries
$max_oids	    = 10;				# max oids for V2/V3 hosts

# required for SNMP V3
$snmp_auth_username   	= "";
$snmp_auth_password   	= "";
$snmp_auth_protocol  	= "";
$snmp_priv_passphrase 	= "";
$snmp_priv_protocol   	= "";
$snmp_context         	= "";

if ($cmd == "index") {
        # retrieve all indices from target
        $return_arr = reindex(cacti_snmp_walk($hostname, $snmp_community,
        $oids["index"], $snmp_version, $snmp_auth_username,
        $snmp_auth_password, $snmp_auth_protocol, $snmp_priv_passphrase, $snmp_priv_protocol,
        $snmp_context, $snmp_port, $snmp_timeout, $snmp_retries, $max_oids, SNMP_POLLER));
 
        # and print each index as a separate line
        for ($i=0;($i<sizeof($return_arr));$i++) {
                print $return_arr[$i] . "\n";
        }

}elseif ($cmd == "query" && isset($query_field)) {
        $arr_index = reindex(cacti_snmp_walk($hostname, $snmp_community,
        $oids["index"], $snmp_version, $snmp_auth_username,
        $snmp_auth_password, $snmp_auth_protocol, $snmp_priv_passphrase, $snmp_priv_protocol,
        $snmp_context, $snmp_port, $snmp_timeout, $snmp_retries, $max_oids, SNMP_POLLER));
        $arr = reindex(cacti_snmp_walk($hostname, $snmp_community,
        $oids[$query_field], $snmp_version, $snmp_auth_username,
        $snmp_auth_password, $snmp_auth_protocol, $snmp_priv_passphrase, $snmp_priv_protocol,
        $snmp_context, $snmp_port, $snmp_timeout, $snmp_retries, $max_oids, SNMP_POLLER));
 
        for ($i=0;($i<sizeof($arr_index));$i++) {
            if(1 === preg_match('/CPE7000/', $arr[$i]))
                print $arr_index[$i] . $xml_delimiter . $arr[$i] . "\n";
            elseif(1 === preg_match('/secs/', $arr[$i]))
                print $arr_index[$i] . $xml_delimiter . preg_replace("/ secs/", "", $arr[$i]) . "\n";
            else
                print $arr_index[$i] . $xml_delimiter . date_to_timeticks($arr[$i]) . "\n";
        }

}elseif ($cmd == "get" && isset($query_field) && isset($query_index)) {
        $result = (cacti_snmp_get($hostname, $snmp_community,
        $oids[$query_field] . ".$query_index", $snmp_version, $snmp_auth_username,
        $snmp_auth_password, $snmp_auth_protocol, $snmp_priv_passphrase, $snmp_priv_protocol,
        $snmp_context, $snmp_port, $snmp_timeout, $snmp_retries, $max_oids, SNMP_POLLER));

        if(1 === preg_match('/CPE7000/', $result))
            print $result;
        elseif(1 === preg_match('/secs/', $result))
            print preg_replace("/ secs/", "", $result);
        else
            print date_to_timeticks($result);

} else {
        print "Invalid use of script query, required parameters:\n\n";
        print "    <hostname> <community> <version> <snmp_port> <timeout> 
                   <max_oids> <auth_user> <auth_passphrase> <auth_proto>
                   <priv_passphrase> <priv_proto> <context> <cmd>\n";
}
 
function reindex($arr) {
        $return_arr = array();
 
        for ($i=0;($i<sizeof($arr));$i++) {
                $return_arr[$i] = $arr[$i]["value"];
        }
 
        return $return_arr;
}

function date_to_timeticks($date) {
    $search  = [" days", " hours", "minutes"];
    $replace = [""];
    $date    = str_replace($search, $replace, $date);
    $date    = explode(" ", $date);
    array_pop($date);

    $day  = $date[0];
    $hour = $date[1];
    $min  = $date[2];

    $h = preg_replace('/0\./', $hour.".", $min / 60);
    $d = preg_replace('/0\./', $day .".", $h   / 24);
    $t = $d * 8640000;

    return $t;
}

function timeticks_convert($timeticks){
    if($timeticks<=0){
        $convertSeconds = "0 Days - 0 Hours - 0 Mins - 0 Secs";
    }else{
        $lntSecs    = intval($timeticks / 100);
        $intDays    = intval($lntSecs / 86400);
        $intHours   = intval(($lntSecs - ($intDays * 86400)) / 3600);
        $intMinutes = intval(($lntSecs - ($intDays * 86400) - ($intHours * 3600)) / 60);
        $intSeconds = intval(($lntSecs - ($intDays * 86400) - ($intHours * 3600) - ($intMinutes * 60)));
        $convertSeconds = "$intDays Days - $intHours Hours - $intMinutes Mins - $intSeconds Secs";
    }
    return $convertSeconds;
}